
# q-expansion of newform 2016.2.s.d, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2016
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [1765, [1, 0]], [1793, [1, 0]], [577, [0, -1]]]
aps_data = [[0, 0], [0, 0], [0, -3], [-1, 3], [5, -5], [2, 0], [-2, 2], [0, 6], [0, 2], [1, 0], [1, -1], [0, -10], [4, 0], [4, 0], [0, -8], [-5, 5], [13, -13], [0, 8], [14, -14], [12, 0], [6, -6], [0, -11], [7, 0], [0, 6], [19, 0], [-6, 6], [0, -8], [0, -13], [-10, 10], [-12, 0], [-7, 0], [0, 9], [22, -22], [14, 0], [0, -18], [-11, 11], [12, -12], [0, 14], [22, 0], [0, -18], [-12, 12], [0, 0], [0, -12], [-17, 17], [6, 0], [-16, 16], [20, 0], [9, 0], [-1, 1], [0, 8], [0, -6], [6, 0], [1, -1], [-17, 0], [0, 14], [-16, 16], [-21, 21], [0, -11], [28, -28], [-30, 0], [-4, 4], [25, 0], [22, 0], [-24, 24], [0, -1], [0, -13], [0, 0], [-3, 0], [12, -12], [-22, 0], [-24, 24], [0, 26], [19, -19], [0, 0], [-8, 0], [0, 38], [-18, 18], [0, -8], [0, -32], [-19, 19], [12, 0], [-12, 0], [-28, 28], [-14, 0], [0, 21], [0, 9], [-6, 0], [0, 29], [2, 0], [36, 0], [0, -36], [-24, 24], [3, -3], [31, 0], [0, -10], [-14, 0], [0, -25], [-40, 40], [0, 26], [0, 24], [-2, 0], [33, -33], [-11, 11], [0, 6], [4, -4], [-7, 7], [-27, 0], [0, -6], [-18, 18], [23, 0], [0, 3], [22, -22], [8, 0], [-32, 32], [29, 0], [-16, 16], [-34, 0], [48, -48], [0, 25], [-12, 0], [0, 0], [-39, 0], [0, 9], [3, -3], [0, 28], [-15, 0], [0, 10], [0, 30], [-17, 0], [0, 4], [50, -50], [26, 0], [0, 15], [-8, 0], [0, -30], [21, 0], [-30, 30], [4, -4], [19, 0], [4, -4], [-38, 0], [0, -41], [-32, 32], [-9, 0], [42, -42], [10, 0], [6, 0], [0, 0], [0, -40], [0, -6], [0, 8], [26, 0], [-16, 0], [0, -38], [-38, 38], [20, 0], [0, -44], [0, 0], [35, 0], [-31, 31], [0, -28], [4, 0], [19, 0], [0, 15], [-52, 52], [-12, 12], [37, -37], [-38, 38], [19, 0], [0, -7], [15, -15], [-48, 0], [0, 18], [3, -3], [-37, 37], [-8, 0], [-40, 0], [57, -57], [28, 0], [0, 44], [0, 53], [-39, 0], [22, 0], [0, -14], [42, -42], [-15, 15], [-28, 28], [42, -42], [0, -19], [-42, 42], [0, -29], [-27, 0], [0, 62], [0, 22], [65, -65], [-54, 54], [-54, 54], [0, -20], [12, 0], [0, 16], [33, -33], [32, 0], [0, 14], [46, -46], [36, 0], [-17, 17], [0, -40], [0, 39], [-30, 0], [-10, 10], [0, 2], [67, 0], [-1, 0], [0, -33], [-26, 26], [0, 26], [23, -23], [66, -66], [0, -18], [-34, 0], [0, -44], [40, 0], [0, -60], [0, -27], [-23, 0], [-20, 0], [0, -46], [-20, 20], [0, -15], [0, -40], [0, 0], [-36, 36], [-37, 0], [-46, 46], [-34, 0], [34, -34], [0, -39], [0, -15], [8, 0], [-22, 0], [-72, 72], [0, 12], [51, -51], [0, -26], [22, 0], [0, 44], [-65, 0], [-12, 12], [72, -72], [12, 0], [-14, 0], [0, 28], [-10, 10], [50, 0], [-41, 41], [0, 13], [-28, 28], [28, -28], [-19, 0], [0, -33], [24, -24], [-53, 0], [26, -26], [-56, 0], [-56, 56], [0, -38], [31, 0], [18, 0], [0, 0], [-3, 3], [0, 24], [-46, 46], [-25, 25], [0, -17], [13, 0], [0, -8], [0, -21], [14, -14], [0, -10], [0, 68], [-52, 52], [5, -5], [-22, 0], [40, 0], [0, 82], [0, 66], [-47, 47], [-41, 0], [-77, 77], [-72, 0], [-14, 14], [-45, 45], [0, 36], [-20, 0], [-26, 0], [61, -61], [0, -8], [-54, 0], [0, -4], [4, 0], [0, 14], [0, 21], [-61, 61]]
